@@ -36,11 +36,15 @@ import itertools
 import logging
 import os
 import platform
+import re
 import sys
 import threading
 import time
+import urllib.parse
 import warnings
 from copy import deepcopy
+from datetime import timedelta
+from typing import Optional, Tuple
 
 import elasticapm
 from elasticapm.conf import Config, VersionedConfig, constants
@@ -102,6 +106,8 @@ class Client(object):
         self.processors = []
         self.filter_exception_types_dict = {}
         self._service_info = None
+        # setting server_version here is mainly used for testing
+        self.server_version = inline.pop("server_version", None)
 
         self.check_python_version()
 
@@ -115,8 +121,7 @@ class Client(object):
         self.config = VersionedConfig(config, version=None)
 
         # Insert the log_record_factory into the logging library
-        # The LogRecordFactory functionality is only available on python 3.2+
-        if compat.PY3 and not self.config.disable_log_record_factory:
+        if not self.config.disable_log_record_factory:
             record_factory = logging.getLogRecordFactory()
             # Only way to know if it's wrapped is to create a log record
             throwaway_record = record_factory(__name__, logging.DEBUG, __file__, 252, "dummy_msg", [], None)
@@ -132,7 +137,7 @@ class Client(object):
         headers = {
             "Content-Type": "application/x-ndjson",
             "Content-Encoding": "gzip",
-            "User-Agent": "elasticapm-python/%s" % elasticapm.VERSION,
+            "User-Agent": self.get_user_agent(),
         }
 
         transport_kwargs = {
@@ -142,12 +147,12 @@ class Client(object):
             "timeout": self.config.server_timeout,
             "processors": self.load_processors(),
         }
-        self._api_endpoint_url = compat.urlparse.urljoin(
+        self._api_endpoint_url = urllib.parse.urljoin(
             self.config.server_url if self.config.server_url.endswith("/") else self.config.server_url + "/",
             constants.EVENTS_API_PATH,
         )
         transport_class = import_string(self.config.transport_class)
-        self._transport = transport_class(self._api_endpoint_url, self, **transport_kwargs)
+        self._transport = transport_class(url=self._api_endpoint_url, client=self, **transport_kwargs)
         self.config.transport = self._transport
         self._thread_managers["transport"] = self._transport
 
@@ -197,7 +202,8 @@ class Client(object):
             self._metrics.register("elasticapm.metrics.sets.breakdown.BreakdownMetricSet")
         if self.config.prometheus_metrics:
             self._metrics.register("elasticapm.metrics.sets.prometheus.PrometheusMetrics")
-        self._thread_managers["metrics"] = self._metrics
+        if self.config.metrics_interval:
+            self._thread_managers["metrics"] = self._metrics
         compat.atexit_register(self.close)
         if self.config.central_config:
             self._thread_managers["config"] = self.config
@@ -278,17 +284,20 @@ class Client(object):
             flush = False
         self._transport.queue(event_type, data, flush)
 
-    def begin_transaction(self, transaction_type, trace_parent=None, start=None):
+    def begin_transaction(self, transaction_type, trace_parent=None, start=None, auto_activate=True):
         """
         Register the start of a transaction on the client
 
         :param transaction_type: type of the transaction, e.g. "request"
         :param trace_parent: an optional TraceParent object for distributed tracing
         :param start: override the start timestamp, mostly useful for testing
+        :param auto_activate: whether to set this transaction in execution_context
         :return: the started transaction object
         """
         if self.config.is_recording:
-            return self.tracer.begin_transaction(transaction_type, trace_parent=trace_parent, start=start)
+            return self.tracer.begin_transaction(
+                transaction_type, trace_parent=trace_parent, start=start, auto_activate=auto_activate
+            )
 
     def end_transaction(self, name=None, result="", duration=None):
         """
@@ -299,6 +308,8 @@ class Client(object):
         :param duration: override duration, mostly useful for testing
         :return: the ended transaction object
         """
+        if duration is not None and not isinstance(duration, timedelta):
+            duration = timedelta(seconds=duration)
         transaction = self.tracer.end_transaction(result, name, duration=duration)
         return transaction
 
@@ -415,6 +426,17 @@ class Client(object):
             self.logger.warning("Unknown value for CLOUD_PROVIDER, skipping cloud metadata: {}".format(provider))
             return {}
 
+    def get_user_agent(self) -> str:
+        """
+        Compiles the user agent, which will be added as a header to all requests
+        to the APM Server
+        """
+        if self.config.service_version:
+            service_version = re.sub(r"[^\t _\x21-\x27\x2a-\x5b\x5d-\x7e\x80-\xff]", "_", self.config.service_version)
+            return "apm-agent-python/{} ({} {})".format(elasticapm.VERSION, self.config.service_name, service_version)
+        else:
+            return "apm-agent-python/{} ({})".format(elasticapm.VERSION, self.config.service_name)
+
     def build_metadata(self):
         data = {
             "service": self.get_service_info(),
@@ -458,6 +480,12 @@ class Client(object):
         event_data["context"] = context
         if transaction and transaction.labels:
             context["tags"] = deepcopy(transaction.labels)
+        # No intake for otel.attributes, so make them labels
+        if "otel_attributes" in context:
+            if context.get("tags"):
+                context["tags"].update(context.pop("otel_attributes"))
+            else:
+                context["tags"] = context.pop("otel_attributes")
 
         # if '.' not in event_type:
         # Assume it's a builtin
@@ -472,7 +500,7 @@ class Client(object):
         if custom.get("culprit"):
             culprit = custom.pop("culprit")
 
-        for k, v in compat.iteritems(result):
+        for k, v in result.items():
             if k not in event_data:
                 event_data[k] = v
 
@@ -504,7 +532,7 @@ class Client(object):
         if "stacktrace" in log and not culprit:
             culprit = stacks.get_culprit(log["stacktrace"], self.config.include_paths, self.config.exclude_paths)
 
-        if "level" in log and isinstance(log["level"], compat.integer_types):
+        if "level" in log and isinstance(log["level"], int):
             log["level"] = logging.getLevelName(log["level"]).lower()
 
         if log:
@@ -531,7 +559,11 @@ class Client(object):
             # parent id might already be set in the handler
             event_data.setdefault("parent_id", span.id if span else transaction.id)
             event_data["transaction_id"] = transaction.id
-            event_data["transaction"] = {"sampled": transaction.is_sampled, "type": transaction.transaction_type}
+            event_data["transaction"] = {
+                "sampled": transaction.is_sampled,
+                "type": transaction.transaction_type,
+                "name": transaction.name,
+            }
 
         return event_data
 
@@ -615,6 +647,22 @@ class Client(object):
         elif v < (3, 5):
             warnings.warn("The Elastic APM agent only supports Python 3.5+", DeprecationWarning)
 
+    def check_server_version(
+        self, gte: Optional[Tuple[int, ...]] = None, lte: Optional[Tuple[int, ...]] = None
+    ) -> bool:
+        """
+        Check APM Server version against greater-or-equal and/or lower-or-equal limits, provided as tuples of integers.
+        If server_version is not set, always returns True.
+        :param gte: a tuple of ints describing the greater-or-equal limit, e.g. (7, 16)
+        :param lte: a tuple of ints describing the lower-or-equal limit, e.g. (7, 99)
+        :return: bool
+        """
+        if not self.server_version:
+            return True
+        gte = gte or (0,)
+        lte = lte or (2 ** 32,)  # let's assume APM Server version will never be greater than 2^32
+        return bool(gte <= self.server_version <= lte)
+
 
 class DummyClient(Client):
     """Sends messages into an empty void"""
@@ -623,11 +671,11 @@ class DummyClient(Client):
         return None
 
 
-def get_client():
+def get_client() -> Client:
     return CLIENT_SINGLETON
 
 
-def set_client(client):
+def set_client(client: Client):
     global CLIENT_SINGLETON
     if CLIENT_SINGLETON:
         logger = get_logger("elasticapm")
