@@ -34,7 +34,7 @@ from starlette.requests import Request
 from starlette.types import Message
 
 from elasticapm.conf import Config, constants
-from elasticapm.utils import compat, get_url_dict
+from elasticapm.utils import get_url_dict
 
 
 async def get_data_from_request(request: Request, config: Config, event_type: str) -> dict:
@@ -50,7 +50,7 @@ async def get_data_from_request(request: Request, config: Config, event_type: st
     """
     result = {
         "method": request.method,
-        "socket": {"remote_address": _get_client_ip(request), "encrypted": request.url.is_secure},
+        "socket": {"remote_address": _get_client_ip(request)},
         "cookies": request.cookies,
     }
     if config.capture_headers:
@@ -86,12 +86,12 @@ async def get_data_from_response(message: dict, config: Config, event_type: str)
     """
     result = {}
 
-    if "status_code" in message:
+    if "status" in message:
         result["status_code"] = message["status"]
 
     if config.capture_headers and "headers" in message:
         headers = Headers(raw=message["headers"])
-        result["headers"] = {key: ";".join(headers.getlist(key)) for key in compat.iterkeys(headers)}
+        result["headers"] = {key: ";".join(headers.getlist(key)) for key in headers.keys()}
 
     return result
 
