@@ -39,7 +39,7 @@ import warnings
 from elasticapm import get_client
 from elasticapm.base import Client
 from elasticapm.traces import execution_context
-from elasticapm.utils import compat, wrapt
+from elasticapm.utils import wrapt
 from elasticapm.utils.encoding import to_unicode
 from elasticapm.utils.stacks import iter_stack_frames
 
@@ -92,7 +92,7 @@ class LoggingHandler(logging.Handler):
     def _emit(self, record, **kwargs):
         data = {}
 
-        for k, v in compat.iteritems(record.__dict__):
+        for k, v in record.__dict__.items():
             if "." not in k and k not in ("culprit",):
                 continue
             data[k] = v
@@ -155,7 +155,7 @@ class LoggingHandler(logging.Handler):
 
         return self.client.capture(
             "Message",
-            param_message={"message": compat.text_type(record.msg), "params": record.args},
+            param_message={"message": str(record.msg), "params": record.args},
             stack=stack,
             custom=custom,
             exception=exception,
@@ -231,15 +231,12 @@ def _add_attributes_to_log_record(record):
     client = get_client()
     service_name = client.config.service_name if client else None
     record.elasticapm_service_name = service_name
-    event_dataset = f"{client.config.service_name}.log" if client else None
-    record.elasticapm_event_dataset = event_dataset
 
     record.elasticapm_labels = {
         "transaction.id": transaction_id,
         "trace.id": trace_id,
         "span.id": span_id,
         "service.name": service_name,
-        "event.dataset": event_dataset,
     }
 
     return record
@@ -268,10 +265,7 @@ class Formatter(logging.Formatter):
             "trace.id=%(elasticapm_trace_id)s "
             "span.id=%(elasticapm_span_id)s"
         )
-        if compat.PY3:
-            super(Formatter, self).__init__(fmt=fmt, datefmt=datefmt, style=style)
-        else:
-            super(Formatter, self).__init__(fmt=fmt, datefmt=datefmt)
+        super(Formatter, self).__init__(fmt=fmt, datefmt=datefmt, style=style)
 
     def format(self, record):
         if not hasattr(record, "elasticapm_transaction_id"):
@@ -279,7 +273,6 @@ class Formatter(logging.Formatter):
             record.elasticapm_trace_id = None
             record.elasticapm_span_id = None
             record.elasticapm_service_name = None
-            record.elasticapm_event_dataset = None
         return super(Formatter, self).format(record=record)
 
     def formatTime(self, record, datefmt=None):
@@ -288,5 +281,4 @@ class Formatter(logging.Formatter):
             record.elasticapm_trace_id = None
             record.elasticapm_span_id = None
             record.elasticapm_service_name = None
-            record.elasticapm_event_dataset = None
         return super(Formatter, self).formatTime(record=record, datefmt=datefmt)
