@@ -29,6 +29,8 @@
 #  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 from __future__ import absolute_import
 
+from typing import Optional, Union
+
 from elasticapm.instrumentation.packages.dbapi2 import (
     ConnectionProxy,
     CursorProxy,
@@ -36,7 +38,7 @@ from elasticapm.instrumentation.packages.dbapi2 import (
     extract_signature,
 )
 from elasticapm.traces import capture_span
-from elasticapm.utils import compat, default_ports
+from elasticapm.utils import default_ports
 
 
 class PGCursorProxy(CursorProxy):
@@ -50,7 +52,7 @@ class PGCursorProxy(CursorProxy):
         if hasattr(sql, "as_string"):
             sql = sql.as_string(self.__wrapped__)
         # if the sql string is already a byte string, we need to decode it using the connection encoding
-        if isinstance(sql, compat.binary_type):
+        if isinstance(sql, bytes):
             sql = sql.decode(psycopg2_extensions.encodings[self.__wrapped__.connection.encoding])
         return sql
 
@@ -76,23 +78,11 @@ class Psycopg2Instrumentation(DbApi2Instrumentation):
     def call(self, module, method, wrapped, instance, args, kwargs):
         signature = "psycopg2.connect"
 
-        host = kwargs.get("host")
-        if host:
-            signature += " " + compat.text_type(host)
-
-            port = kwargs.get("port")
-            if port:
-                port = str(port)
-                if int(port) != default_ports.get("postgresql"):
-                    host += ":" + port
-            signature += " " + compat.text_type(host)
-        else:
-            # Parse connection string and extract host/port
-            pass
+        host, port = get_destination_info(kwargs.get("host"), kwargs.get("port"))
+        signature = f"{signature} {host}:{port}"
         destination_info = {
-            "address": kwargs.get("host", "localhost"),
-            "port": int(kwargs.get("port", default_ports.get("postgresql"))),
-            "service": {"name": "postgresql", "resource": "postgresql", "type": "db"},
+            "address": host,
+            "port": port,
         }
         with capture_span(
             signature,
@@ -141,3 +131,19 @@ class Psycopg2ExtensionsInstrumentation(DbApi2Instrumentation):
                 kwargs["scope"] = kwargs["scope"].__wrapped__
 
         return wrapped(*args, **kwargs)
+
+
+def get_destination_info(host: Optional[str], port: Union[None, str, int]) -> tuple:
+    if host:
+        if "," in host:  # multiple hosts defined, take first
+            host = host.split(",")[0]
+    else:
+        host = "localhost"
+    if port:
+        port = str(port)
+        if "," in port:  # multiple ports defined, take first
+            port = port.split(",")[0]
+        port = int(port)
+    else:
+        port = default_ports.get("postgresql")
+    return host, port
