@@ -33,7 +33,6 @@ import time
 from collections import defaultdict
 
 from elasticapm.conf import constants
-from elasticapm.utils import compat
 from elasticapm.utils.logging import get_logger
 from elasticapm.utils.module_import import import_string
 from elasticapm.utils.threading import IntervalTimer, ThreadManager
@@ -69,7 +68,7 @@ class MetricsRegistry(ThreadManager):
                 class_obj = import_string(class_path)
                 self._metricsets[class_path] = class_obj(self)
             except ImportError as e:
-                logger.warning("Could not register %s metricset: %s", class_path, compat.text_type(e))
+                logger.warning("Could not register %s metricset: %s", class_path, str(e))
 
     def get_metricset(self, class_path):
         try:
@@ -85,7 +84,7 @@ class MetricsRegistry(ThreadManager):
         if self.client.config.is_recording:
             logger.debug("Collecting metrics")
 
-            for _, metricset in compat.iteritems(self._metricsets):
+            for _, metricset in self._metricsets.items():
                 for data in metricset.collect():
                     self.client.queue(constants.METRICSET, data)
 
@@ -108,7 +107,7 @@ class MetricsRegistry(ThreadManager):
 
     @property
     def collect_interval(self):
-        return self.client.config.metrics_interval / 1000.0
+        return self.client.config.metrics_interval.total_seconds()
 
     @property
     def ignore_patterns(self):
@@ -121,6 +120,7 @@ class MetricsSet(object):
         self._counters = {}
         self._gauges = {}
         self._timers = {}
+        self._histograms = {}
         self._registry = registry
         self._label_limit_logged = False
 
@@ -155,7 +155,10 @@ class MetricsSet(object):
         """
         return self._metric(self._timers, Timer, name, reset_on_collect, labels, unit)
 
-    def _metric(self, container, metric_class, name, reset_on_collect, labels, unit=None):
+    def histogram(self, name, reset_on_collect=False, unit=None, buckets=None, **labels):
+        return self._metric(self._histograms, Histogram, name, reset_on_collect, labels, unit, buckets=buckets)
+
+    def _metric(self, container, metric_class, name, reset_on_collect, labels, unit=None, **kwargs):
         """
         Returns an existing or creates and returns a metric
         :param container: the container for the metric
@@ -172,7 +175,10 @@ class MetricsSet(object):
             if key not in container:
                 if any(pattern.match(name) for pattern in self._registry.ignore_patterns):
                     metric = noop_metric
-                elif len(self._gauges) + len(self._counters) + len(self._timers) >= DISTINCT_LABEL_LIMIT:
+                elif (
+                    len(self._gauges) + len(self._counters) + len(self._timers) + len(self._histograms)
+                    >= DISTINCT_LABEL_LIMIT
+                ):
                     if not self._label_limit_logged:
                         self._label_limit_logged = True
                         logger.warning(
@@ -181,7 +187,7 @@ class MetricsSet(object):
                         )
                     metric = noop_metric
                 else:
-                    metric = metric_class(name, reset_on_collect=reset_on_collect, unit=unit)
+                    metric = metric_class(name, reset_on_collect=reset_on_collect, unit=unit, **kwargs)
                 container[key] = metric
             return container[key]
 
@@ -202,35 +208,67 @@ class MetricsSet(object):
         samples = defaultdict(dict)
         if self._counters:
             # iterate over a copy of the dict to avoid threading issues, see #717
-            for (name, labels), c in compat.iteritems(self._counters.copy()):
-                if c is not noop_metric:
-                    val = c.val
-                    if val or not c.reset_on_collect:
+            for (name, labels), counter in self._counters.copy().items():
+                if counter is not noop_metric:
+                    val = counter.val
+                    if val or not counter.reset_on_collect:
                         samples[labels].update({name: {"value": val}})
-                    if c.reset_on_collect:
-                        c.reset()
+                    if counter.reset_on_collect:
+                        counter.reset()
         if self._gauges:
-            for (name, labels), g in compat.iteritems(self._gauges.copy()):
-                if g is not noop_metric:
-                    val = g.val
-                    if val or not g.reset_on_collect:
-                        samples[labels].update({name: {"value": val}})
-                    if g.reset_on_collect:
-                        g.reset()
+            for (name, labels), gauge in self._gauges.copy().items():
+                if gauge is not noop_metric:
+                    val = gauge.val
+                    if val or not gauge.reset_on_collect:
+                        samples[labels].update({name: {"value": val, "type": "gauge"}})
+                    if gauge.reset_on_collect:
+                        gauge.reset()
         if self._timers:
-            for (name, labels), t in compat.iteritems(self._timers.copy()):
-                if t is not noop_metric:
-                    val, count = t.val
-                    if val or not t.reset_on_collect:
+            for (name, labels), timer in self._timers.copy().items():
+                if timer is not noop_metric:
+                    val, count = timer.val
+                    if val or not timer.reset_on_collect:
                         sum_name = ".sum"
-                        if t._unit:
-                            sum_name += "." + t._unit
+                        if timer._unit:
+                            sum_name += "." + timer._unit
                         samples[labels].update({name + sum_name: {"value": val}})
                         samples[labels].update({name + ".count": {"value": count}})
-                    if t.reset_on_collect:
-                        t.reset()
+                    if timer.reset_on_collect:
+                        timer.reset()
+        if self._histograms:
+            for (name, labels), histo in self._histograms.copy().items():
+                if histo is not noop_metric:
+                    counts = histo.val
+                    if counts or not histo.reset_on_collect:
+                        # For the bucket values, we follow the approach described by Prometheus's
+                        # histogram_quantile function
+                        # (https://prometheus.io/docs/prometheus/latest/querying/functions/#histogram_quantile)
+                        # to achieve consistent percentile aggregation results:
+                        #
+                        # "The histogram_quantile() function interpolates quantile values by assuming a linear
+                        # distribution within a bucket. (...) If a quantile is located in the highest bucket,
+                        # the upper bound of the second highest bucket is returned. A lower limit of the lowest
+                        # bucket is assumed to be 0 if the upper bound of that bucket is greater than 0. In that
+                        # case, the usual linear interpolation is applied within that bucket. Otherwise, the upper
+                        # bound of the lowest bucket is returned for quantiles located in the lowest bucket."
+                        bucket_midpoints = []
+                        for i, bucket_le in enumerate(histo.buckets):
+                            if i == 0:
+                                if bucket_le > 0:
+                                    bucket_le /= 2.0
+                            elif i == len(histo.buckets) - 1:
+                                bucket_le = histo.buckets[i - 1]
+                            else:
+                                bucket_le = histo.buckets[i - 1] + (bucket_le - histo.buckets[i - 1]) / 2.0
+                            bucket_midpoints.append(bucket_le)
+                        samples[labels].update(
+                            {name: {"counts": counts, "values": bucket_midpoints, "type": "histogram"}}
+                        )
+                    if histo.reset_on_collect:
+                        histo.reset()
+
         if samples:
-            for labels, sample in compat.iteritems(samples):
+            for labels, sample in samples.items():
                 result = {"samples": sample, "timestamp": timestamp}
                 if labels:
                     result["tags"] = {k: v for k, v in labels}
@@ -247,7 +285,7 @@ class MetricsSet(object):
         return data
 
     def _labels_to_key(self, labels):
-        return tuple((k, compat.text_type(v)) for k, v in sorted(compat.iteritems(labels)))
+        return tuple((k, str(v)) for k, v in sorted(labels.items()))
 
 
 class SpanBoundMetricSet(MetricsSet):
@@ -263,8 +301,16 @@ class SpanBoundMetricSet(MetricsSet):
         return data
 
 
-class Counter(object):
-    __slots__ = ("name", "_lock", "_initial_value", "_val", "reset_on_collect")
+class BaseMetric(object):
+    __slots__ = ("name", "reset_on_collect")
+
+    def __init__(self, name, reset_on_collect=False, **kwargs):
+        self.name = name
+        self.reset_on_collect = reset_on_collect
+
+
+class Counter(BaseMetric):
+    __slots__ = BaseMetric.__slots__ + ("_lock", "_initial_value", "_val")
 
     def __init__(self, name, initial_value=0, reset_on_collect=False, unit=None):
         """
@@ -273,10 +319,9 @@ class Counter(object):
         :param initial_value: initial value of the counter, defaults to 0
         :param unit: unit of the observed counter. Unused for counters
         """
-        self.name = name
         self._lock = threading.Lock()
         self._val = self._initial_value = initial_value
-        self.reset_on_collect = reset_on_collect
+        super(Counter, self).__init__(name, reset_on_collect=reset_on_collect)
 
     def inc(self, delta=1):
         """
@@ -318,8 +363,8 @@ class Counter(object):
             self._val = value
 
 
-class Gauge(object):
-    __slots__ = ("name", "_val", "reset_on_collect")
+class Gauge(BaseMetric):
+    __slots__ = BaseMetric.__slots__ + ("_val",)
 
     def __init__(self, name, reset_on_collect=False, unit=None):
         """
@@ -327,9 +372,8 @@ class Gauge(object):
         :param name: label of the gauge
         :param unit of the observed gauge. Unused for gauges
         """
-        self.name = name
         self._val = None
-        self.reset_on_collect = reset_on_collect
+        super(Gauge, self).__init__(name, reset_on_collect=reset_on_collect)
 
     @property
     def val(self):
@@ -343,16 +387,15 @@ class Gauge(object):
         self._val = 0
 
 
-class Timer(object):
-    __slots__ = ("name", "_val", "_count", "_lock", "_unit", "reset_on_collect")
+class Timer(BaseMetric):
+    __slots__ = BaseMetric.__slots__ + ("_val", "_count", "_lock", "_unit")
 
     def __init__(self, name=None, reset_on_collect=False, unit=None):
-        self.name = name
-        self._val = 0
-        self._count = 0
+        self._val: float = 0
+        self._count: int = 0
         self._unit = unit
         self._lock = threading.Lock()
-        self.reset_on_collect = reset_on_collect
+        super(Timer, self).__init__(name, reset_on_collect=reset_on_collect)
 
     def update(self, duration, count=1):
         with self._lock:
@@ -375,6 +418,46 @@ class Timer(object):
             self._val, self._count = value
 
 
+class Histogram(BaseMetric):
+    DEFAULT_BUCKETS = (0.005, 0.01, 0.025, 0.05, 0.075, 0.1, 0.25, 0.5, 0.75, 1, 2.5, 5, 7.5, 10, float("inf"))
+
+    __slots__ = BaseMetric.__slots__ + ("_lock", "_buckets", "_counts", "_lock", "_unit")
+
+    def __init__(self, name=None, reset_on_collect=False, unit=None, buckets=None):
+        self._lock = threading.Lock()
+        self._buckets = buckets or Histogram.DEFAULT_BUCKETS
+        if self._buckets[-1] != float("inf"):
+            self._buckets.append(float("inf"))
+        self._counts = [0] * len(self._buckets)
+        self._unit = unit
+        super(Histogram, self).__init__(name, reset_on_collect=reset_on_collect)
+
+    def update(self, value, count=1):
+        pos = 0
+        while value > self._buckets[pos]:
+            pos += 1
+        with self._lock:
+            self._counts[pos] += count
+
+    @property
+    def val(self):
+        with self._lock:
+            return self._counts
+
+    @val.setter
+    def val(self, value):
+        with self._lock:
+            self._counts = value
+
+    @property
+    def buckets(self):
+        return self._buckets
+
+    def reset(self):
+        with self._lock:
+            self._counts = [0] * len(self._buckets)
+
+
 class NoopMetric(object):
     """
     A no-op metric that implements the "interface" of both Counter and Gauge.
