@@ -32,10 +32,7 @@
 
 import atexit
 import functools
-import operator
 import platform
-import sys
-import types
 
 
 def noop_decorator(func):
@@ -69,79 +66,6 @@ def atexit_register(func):
         atexit.register(func)
 
 
-# Compatibility layer for Python2/Python3, partly inspired by /modified from six, https://github.com/benjaminp/six
-# Remainder of this file: Copyright Benjamin Peterson, MIT License https://github.com/benjaminp/six/blob/master/LICENSE
-
-PY2 = sys.version_info[0] == 2
-PY3 = sys.version_info[0] == 3
-
-
-if PY2:
-    from urllib import getproxies_environment, proxy_bypass_environment  # noqa F401
-
-    import Queue as queue  # noqa F401
-    import StringIO
-    import urlparse  # noqa F401
-    from urllib2 import HTTPError  # noqa F401
-
-    StringIO = BytesIO = StringIO.StringIO
-
-    string_types = (basestring,)  # noqa F821
-    integer_types = (int, long)  # noqa F821
-    class_types = (type, types.ClassType)
-    text_type = unicode  # noqa F821
-    binary_type = str
-    irange = xrange  # noqa F821
-
-    def b(s):
-        return s
-
-    get_function_code = operator.attrgetter("func_code")
-
-    def iterkeys(d, **kwargs):
-        return d.iterkeys(**kwargs)
-
-    def iteritems(d, **kwargs):
-        return d.iteritems(**kwargs)
-
-    # for django.utils.datastructures.MultiValueDict
-    def iterlists(d, **kw):
-        return d.iterlists(**kw)
-
-
-else:
-    import io
-    import queue  # noqa F401
-    from urllib import parse as urlparse  # noqa F401
-    from urllib.error import HTTPError  # noqa F401
-    from urllib.request import getproxies_environment, proxy_bypass_environment  # noqa F401
-
-    StringIO = io.StringIO
-    BytesIO = io.BytesIO
-
-    string_types = (str,)
-    integer_types = (int,)
-    class_types = (type,)
-    text_type = str
-    binary_type = bytes
-    irange = range
-
-    def b(s):
-        return s.encode("latin-1")
-
-    get_function_code = operator.attrgetter("__code__")
-
-    def iterkeys(d, **kwargs):
-        return iter(d.keys(**kwargs))
-
-    def iteritems(d, **kwargs):
-        return iter(d.items(**kwargs))
-
-    # for django.utils.datastructures.MultiValueDict
-    def iterlists(d, **kw):
-        return iter(d.lists(**kw))
-
-
 def get_default_library_patters():
     """
     Returns library paths depending on the used platform.
@@ -169,7 +93,7 @@ def multidict_to_dict(d):
     :param d: a MultiDict or MultiValueDict instance
     :return: a dict instance
     """
-    return dict((k, v[0] if len(v) == 1 else v) for k, v in iterlists(d))
+    return dict((k, v[0] if len(v) == 1 else v) for k, v in d.lists())
 
 
 try:
